/*
 * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * The contents of this file constitute Original Code as defined in and
 * are subject to the Apple Public Source License Version 1.1 (the
 * "License").  You may not use this file except in compliance with the
 * License.  Please obtain a copy of the License at
 * http://www.apple.com/publicsource and read it before using this file.
 * 
 * This Original Code and all software distributed under the License are
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
#ifndef __HFS_ENDIAN_H__
#define __HFS_ENDIAN_H__

/*
 * hfs_endian.h
 *
 * This file prototypes endian swapping routines for the HFS/HFS Plus
 * volume format.
 */
#include <hfs/hfs_format.h>
#include <architecture/byte_order.h>

#if 0
typedef struct {
	u_int16_t u16;
} __packed u_big_int16_t;

typedef struct {
	u_int32_t u32;
} __packed u_big_int32_t;

typedef struct {
	u_int64_t u64;
} __packed u_big_int64_t;

    #define SWAP_BE16(__a) 							NXSwapBigShortToHost ((__a).u16)
	#define SET_SWAP_BE16(__d, __a)					(__d).u16 = NXSwapBigShortToHost(__a)
    #define SWAP_BE32(__a) 							NXSwapBigLongToHost ((__a).u32)
	#define SET_SWAP_BE32(__d, __a)					(__d).u32 = NXSwapBigLongToHost(__a)
    #define SWAP_BE64(__a) 							NXSwapBigLongLongToHost ((__a).u64)
    #define SET_SWAP_BE64(__d, __a) 				(__d).u64 = NXSwapBigLongLongToHost (__a)
#else
typedef u_int16_t u_big_int16_t;
typedef u_int32_t u_big_int32_t;
typedef u_int64_t u_big_int64_t;

    #define SWAP_BE16(__a) 							NXSwapBigShortToHost (__a)
	#define SET_SWAP_BE16(__d, __a)					(__d) = NXSwapBigShortToHost(__a)
    #define SWAP_BE32(__a) 							NXSwapBigLongToHost (__a)
	#define SET_SWAP_BE32(__d, __a)					(__d) = NXSwapBigLongToHost(__a)
    #define SWAP_BE64(__a) 							NXSwapBigLongLongToHost (__a)
    #define SET_SWAP_BE64(__d, __a) 				(__d) = NXSwapBigLongLongToHost (__a)
#endif

#ifdef __cplusplus
extern "C" {
#endif

void hfs_swap_HFSMasterDirectoryBlock (void *buf);
void hfs_swap_HFSPlusVolumeHeader (void *buf);

#ifdef __cplusplus
}
#endif

#endif /* __HFS_FORMAT__ */
